module ExpressAnalytics
  class DailyStatistic < ActiveRecord::Base
    include ActionView::Helpers::NumberHelper

    def self.all_statistics
      Dir.glob(File.join(Rails.root, 'app', 'statistics', '*.rb')).map do |statistic|
        file_name = File.basename(statistic).gsub('.rb', '')
        class_name = file_name.classify
        class_name = class_name.pluralize if file_name.match(/s$/)
        class_name.constantize
      end
    end

    def self.calculate_all!(date=nil)
      all_statistics.each { |stat| stat.calculate!(date) }
    end

    def self.calculate(&block)
      define_method(calculate_method_name, &block)
    end

    def self.calculate!(date_or_date_range = nil)
      _arrayify(date_or_date_range||Date.today).each do |date|
        _calculate_for_date!(date)
      end
    end

    def calculate!
      self.value = send(self.class.calculate_method_name, calculated_for)
    end

    def value_formatted
      self.value
    end

    def self.aggregate_for(range)
      aggregate_record = where(calculated_for: range).select("#{aggregation_operation.to_s.upcase}(value) AS value").order(nil).first
      aggregate_record.value
    end

    def self.aggregation_operation
      :sum
    end

    private

      def self._arrayify(date_or_date_range)
        if date_or_date_range.present? &&
           date_or_date_range.kind_of?(Range)
          date_or_date_range.to_a
        else
          [date_or_date_range].flatten.compact
        end
      end

      def self._calculate_for_date!(date)
        stat = find_by_calculated_for(date.to_date) || new(calculated_for: date.to_date)
        stat.calculate!
        stat.save
        stat
      end

      def self.calculate_method_name
        "calculate_#{self.class.to_s.underscore}!".to_sym
      end

  end
end
